/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package org.glite.ce.cream;

import java.io.File;
import java.io.IOException;
import java.rmi.RemoteException;

import org.apache.axis.MessageContext;
import org.glite.ce.cream.CREAMService;
import org.glite.ce.cream.configuration.CREAMConfigConstants;
import org.glite.ce.creamapij.Job;
import org.glite.ce.creamapij.faults.GridProxyDelegationFault;
import org.glite.ce.creamapij.faults.InvalidArgumentFault;
import org.glite.ce.creamapij.faults.JobStatusInvalidFault;
import org.glite.ce.creamapij.faults.JobUnknownFault;
import org.glite.ce.creamapij.faults.NoSuitableResourcesFault;
import org.glite.ce.creamapij.faults.OperationNotSupportedFault;
import org.glite.ce.creamapij.types.JobFilter;
import org.glite.ce.creamapij.types.JobIdList;
import org.glite.ce.creamapij.types.JobInfo;
import org.glite.ce.creamapij.types.JobInfoList;
import org.glite.ce.faults.AuthenticationFault;
import org.glite.ce.faults.AuthorizationFault;
import org.glite.ce.faults.GenericFault;

import junit.framework.TestCase;

public class CREAMServiceTest extends TestCase {

	MessageContext messageContext = MessageContext.getCurrentContext();
	public String firstJobID ;
	public String secondJobID ;
	private CREAMService creamService = new CREAMService();
	

	public void testCREAMService() {
		assertNotNull(creamService);
	}
	

	public void testGetProperty() {
		assertEquals("TestConnector.jar",creamService.getProperty(CREAMConfigConstants.CONNECTOR_FILE_LABEL));
	}


	public void testInitLRMSConnectorLoader() {
	}

	public void testInitJobInfoTable() {
	}

	public void testInitCREAMJournalManager() {
	}

	public void testInitLRMSConnector() {
	}
	

	public void testMakeLRMSConnectorClass() {
		// null jar file
		try{
			creamService.makeLRMSConnectorClass(null);			
		} catch(Exception e){
			assertEquals("The jar file is null!", e.getMessage());
		}
		
		File jarFile = new File("test");
		try{
			
			creamService.makeLRMSConnectorClass(jarFile);			
		} catch(Exception e){
			assertEquals("Cannot create LRMSConnectorClass from file " + jarFile.getName() +  ". Reason is: "+"Cannot create a LRMSConnector instance from file \"" + jarFile.getAbsolutePath() + "\"", e.getMessage());
		}
	}


	public void testJobRegister() {
		String jdl ="[\nVirtualOrganisation = \"topolino\";" +
		"\nrequirements =  other.GlueCEStateStatus == \"Production\";" +
		"\nRank = -other.GlueCEStateEstimatedResponseTime;" +
		"\nexecutable=\"/bin/sleep\";" +
		"\narguments=\"60\";" +
		"\nBatchSystem=\"lsf\";" +
		"\nQueueName=\"grid02\"" +
		"\n]";
		String faultMessage = "";
		String faultCause = "";
		JobInfo jobInfo = null;

		try { // null jdl
			jobInfo = creamService.jobRegister("",null,false,0);
		} catch (AuthorizationFault e) {
			e.printStackTrace();
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (GridProxyDelegationFault e) {
			e.printStackTrace();
		} catch (InvalidArgumentFault e) {
			assertEquals("Invalid argument",e.getDescription());
			assertEquals("jdl == null",e.getFaultCause());
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		
		try { // invalid jdl
			jobInfo = creamService.jobRegister("","fake_jdl",false,0);
		} catch (AuthorizationFault e) {
			e.printStackTrace();
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (GridProxyDelegationFault e) {
			e.printStackTrace();
		} catch (InvalidArgumentFault e) {
			assertEquals("JDL error",e.getDescription());
			assertEquals("Unable to parse: doesn't seem to be a valid Expression",e.getFaultCause());
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		
		try {
			jobInfo = creamService.jobRegister("",jdl,false,0);
		} catch (AuthorizationFault e) {
			e.printStackTrace();
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (GridProxyDelegationFault e) {
			e.printStackTrace();
		} catch (InvalidArgumentFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		assertNotNull(jobInfo);
		assertEquals("REGISTERED",jobInfo.getStatus());
		firstJobID = jobInfo.getCREAMJobId();
		try {
			creamService.jobSuspend(new JobIdList(new String[]{firstJobID}));
		} catch (AuthenticationFault e1) {
			e1.printStackTrace();
		} catch (JobUnknownFault e1) {
			e1.printStackTrace();
		} catch (OperationNotSupportedFault e1) {
			e1.printStackTrace();
		} catch (GenericFault e1) {
			assertEquals("\nThe following jobs have a status not compatible with the jobSuspend command: "+firstJobID,e1.getDescription());
		} catch (JobStatusInvalidFault e1) {
			e1.printStackTrace();
		} catch (AuthorizationFault e1) {
			e1.printStackTrace();
		} catch (InvalidArgumentFault e1) {
			e1.printStackTrace();
		} catch (RemoteException e1) {
			e1.printStackTrace();
		}
		
		try {
			creamService.jobResume(new JobIdList(new String[]{firstJobID}));
		} catch (AuthenticationFault e1) {
			e1.printStackTrace();
		} catch (JobUnknownFault e1) {
			e1.printStackTrace();
		} catch (OperationNotSupportedFault e1) {
			e1.printStackTrace();
		} catch (GenericFault e1) {
			assertEquals("\nThe following jobs have a status not compatible with the jobResume command: "+firstJobID,e1.getDescription());
		} catch (JobStatusInvalidFault e1) {
			e1.printStackTrace();
		} catch (AuthorizationFault e1) {
			e1.printStackTrace();
		} catch (InvalidArgumentFault e1) {
			e1.printStackTrace();
		} catch (RemoteException e1) {
			e1.printStackTrace();
		}
//		starting job
		try {
			creamService.jobStart(firstJobID);
		} catch (JobStatusInvalidFault e) {
			e.printStackTrace();
		} catch (JobUnknownFault e) {
			e.printStackTrace();
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (AuthorizationFault e) {
			e.printStackTrace();
		} catch (InvalidArgumentFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		
//		starting already started job
		try {
			creamService.jobStart(firstJobID);
		} catch (JobStatusInvalidFault e) {
			assertEquals("job "+firstJobID+" already started (current status=PENDING)",e.getDescription());
		} catch (JobUnknownFault e) {
			e.printStackTrace();
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (AuthorizationFault e) {
			e.printStackTrace();
		} catch (InvalidArgumentFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		
		
		// with autostart
		try {
			jobInfo = creamService.jobRegister("",jdl,true,0);
		} catch (AuthorizationFault e) {
			e.printStackTrace();
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (GridProxyDelegationFault e) {
			e.printStackTrace();
		} catch (InvalidArgumentFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}

		secondJobID = jobInfo.getCREAMJobId();

// 		getting jobInfo about the started job
		String[] jobIDs = new String []{firstJobID,secondJobID};
		JobFilter jobFilter = new JobFilter(null,jobIDs,null,null);
		JobInfoList jobInfoList = null;
		try {
			jobInfoList = creamService.jobInfo(jobFilter);
		} catch (AuthenticationFault e) {
			e.printStackTrace();
		} catch (JobUnknownFault e) {
			e.printStackTrace();
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (AuthorizationFault e) {
			e.printStackTrace();
		} catch (InvalidArgumentFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		jobInfo = jobInfoList.getJobInfo(0);
		assertEquals(firstJobID,jobInfo.getCREAMJobId());
		assertEquals("PENDING",jobInfo.getStatus());
		jobInfo = jobInfoList.getJobInfo(1);
		assertEquals(secondJobID,jobInfo.getCREAMJobId());
		assertEquals("PENDING",jobInfo.getStatus());
		

		JobIdList jobList = new JobIdList(new String[]{firstJobID,secondJobID});
		try {
			creamService.jobProxyRenew(null,jobList);
		} catch (AuthorizationFault e) {
			e.printStackTrace();
		} catch (GenericFault e) {
			assertEquals("\nThe following jobs were not found: test",e.getDescription());
		} catch (GridProxyDelegationFault e) {
			e.printStackTrace();
		} catch (InvalidArgumentFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
//		System.out.println("registered job "+firstJobID+" & "+secondJobID);
		
		try {
			creamService.jobPurge(new JobIdList(new String[]{firstJobID}));
		} catch (AuthenticationFault e1) {
			e1.printStackTrace();
		} catch (JobUnknownFault e1) {
			e1.printStackTrace();
		} catch (GenericFault e1) {
			assertEquals("\nThe following jobs have a status not compatible with the jobPurge command: "+firstJobID,e1.getDescription());
		} catch (AuthorizationFault e1) {
			e1.printStackTrace();
		} catch (InvalidArgumentFault e1) {
			e1.printStackTrace();
		} catch (RemoteException e1) {
			e1.printStackTrace();
		}
		
		try {
			creamService.jobCancel(new JobIdList(new String[]{firstJobID,secondJobID}));
		} catch (JobUnknownFault e) {
			e.printStackTrace();
		} catch (JobStatusInvalidFault e) {
			e.printStackTrace();
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (InvalidArgumentFault e) {
			e.printStackTrace();
		} catch (AuthorizationFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		try {
			jobInfoList = creamService.jobInfo(jobFilter);
		} catch (AuthenticationFault e) {
			e.printStackTrace();
		} catch (JobUnknownFault e) {
			e.printStackTrace();
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (AuthorizationFault e) {
			e.printStackTrace();
		} catch (InvalidArgumentFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		
		jobInfo = jobInfoList.getJobInfo(0);
		assertEquals(firstJobID,jobInfo.getCREAMJobId());
		assertEquals("CANCELLED",jobInfo.getStatus());
		

		try {
			creamService.jobPurge(new JobIdList(new String[]{firstJobID,secondJobID}));
		} catch (AuthenticationFault e1) {
			e1.printStackTrace();
		} catch (JobUnknownFault e1) {
			e1.printStackTrace();
		} catch (GenericFault e1) {
			e1.printStackTrace();
		} catch (AuthorizationFault e1) {
			e1.printStackTrace();
		} catch (InvalidArgumentFault e1) {
			e1.printStackTrace();
		} catch (RemoteException e1) {
			e1.printStackTrace();
		}

		try {
			jobInfoList = creamService.jobInfo(jobFilter);
		} catch (AuthenticationFault e) {
			e.printStackTrace();
		} catch (JobUnknownFault e) {
			e.printStackTrace();
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (AuthorizationFault e) {
			e.printStackTrace();
		} catch (InvalidArgumentFault e) {
			e.printStackTrace();
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		
		JobInfo[] jobInfos = jobInfoList.getJobInfo();
		assertEquals(0,jobInfos.length);
		
		
	}

	public void testJobStart() {
		try {
         String jobId = null;
			creamService.jobStart(jobId);
		} catch (JobStatusInvalidFault e) {
		} catch (JobUnknownFault e) {
		} catch (GenericFault e) {
		} catch (AuthorizationFault e) {
		} catch (InvalidArgumentFault e) {
			assertEquals("Invalid argument",e.getDescription());
		} catch (RemoteException e) {
		}
		try {
			creamService.jobStart("not existing job");
		} catch (JobStatusInvalidFault e) {
		} catch (JobUnknownFault e) {
		} catch (GenericFault e) {
		} catch (AuthorizationFault e) {
		} catch (InvalidArgumentFault e) {
			assertEquals("Invalid argument",e.getDescription());
		} catch (RemoteException e) {
		}
		
		
	}

	public void testJobCancel() {
		try {
			creamService.jobCancel(null);
		} catch (JobUnknownFault e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (JobStatusInvalidFault e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (GenericFault e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvalidArgumentFault e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (AuthorizationFault e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		

		JobIdList jobList = new JobIdList(new String[]{"TestJobID"});
		try {
			creamService.jobCancel(jobList);
		} catch (JobUnknownFault e) {
		} catch (JobStatusInvalidFault e) {
		} catch (GenericFault e) {
			assertEquals("\nThe following jobs were not found: "+jobList.getJobID(0),e.getDescription());
		} catch (InvalidArgumentFault e) {
		} catch (AuthorizationFault e) {
		} catch (RemoteException e) {
		}
	}

	public void testJobList() {
		JobIdList jobList = null;
		try {
			jobList = creamService.jobList();
		} catch (GenericFault e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		assertNotNull(jobList);
	}

	public void testPing() {
		try {
			creamService.ping();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void testJobAssess() {
		try {
			creamService.jobAssess("test","test");
		} catch (NoSuitableResourcesFault e) {
		} catch (GenericFault e) {
		} catch (OperationNotSupportedFault e) {
		} catch (InvalidArgumentFault e) {
		} catch (RemoteException e) {
		}
	}

	public void testJobProxyRenew() {
		try {
			creamService.jobProxyRenew(null,null);
		} catch (AuthorizationFault e) {
			e.printStackTrace();
		} catch (GenericFault e) {
			e.printStackTrace();
		} catch (GridProxyDelegationFault e) {
			e.printStackTrace();
		} catch (InvalidArgumentFault e) {
			assertEquals("Invalid argument: the jobIdList is empty",e.getDescription());
		} catch (RemoteException e) {
			e.printStackTrace();
		}
		
//		
//		try {
//			creamService.jobProxyRenew(null,new JobIdList(new String[]{firstJobID}));
//		} catch (AuthorizationFault e) {
//			e.printStackTrace();
//		} catch (GenericFault e) {
//			assertEquals("\nThe following jobs were not found: test",e.getDescription());
//		} catch (GridProxyDelegationFault e) {
//			e.printStackTrace();
//		} catch (InvalidArgumentFault e) {
//			e.printStackTrace();
//		} catch (RemoteException e) {
//			e.printStackTrace();
//		}
		
		/*try {
			creamService.jobProxyRenew(null,new JobIdList(new String[]{"ciao"}));
		} catch (Exception e){
			e.printStackTrace();
		}*/
		
	}


	public void testInitWorkingDir(){
/*
		try {
			creamService.initWorkingDir();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		CREAMUtils.deleteDir((String)messageContext.getProperty(CREAMConfigHandler.CREAM_SCRATCH_DIR));
		CREAMUtils.deleteDir((String)messageContext.getProperty(CREAMConfigHandler.SANDBOX_DIR));
*/
	}

}
