/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Silvano Squizzato <silvano.squizzato@pd.infn.it>
 * 
 */

package org.glite.ce.cream.cmdmanagement.queue.db;

import org.glite.ce.common.db.DatabaseException;
import org.glite.ce.cream.cmdmanagement.db.CommandDBManager;
import org.glite.ce.creamapi.cmdmanagement.CommandException;
import org.glite.ce.creamapi.cmdmanagement.queue.CommandQueueException;

/**
 * Class to test command in the dB command queue.
 */
public final class CommandQueueFetchTest {
    
	public static void main(String[] args) throws Exception {
	    test1();
	}
	
    /**
     * Tests multithreaded access to the command queue for reading and writing on it.
     * Fetcher threads listen for pending commands in the queue.
     * An Enqueuer thread operates the insertion of new commands in the queue. 
     * @throws DatabaseException
     * @throws IllegalArgumentException
     * @throws CommandQueueException 
     * @throws InterruptedException 
     * @throws CommandException 
     */
    private static void test1() throws DatabaseException, IllegalArgumentException, CommandQueueException, InterruptedException, CommandException {
        CommandDBManager cmdManager = new CommandDBManager();
        CommandQueueDBManager queueManager = new CommandQueueDBManager ("JOB_MANAGEMENT");

        System.out.println("BEGIN");
        Thread t3 = new Thread(new Enqueuer(cmdManager, queueManager));
        t3.start();
        
        System.out.println("SLEEPING ...");
        Thread.sleep(60000);
        System.out.println("AWAKEN ...");
                
        Thread t1 = new Thread(new Fetcher(queueManager));
        System.out.println("New Fetcher thread 1 starts with name = " + t1.getName());
        t1.start();
//        Thread.currentThread().sleep(5000);
//        t1.interrupt();
        System.out.println("New Fetcher thread 1 runs with name = " + t1.getName());
        
        Thread t2 = new Thread(new Fetcher(queueManager));
        System.out.println("New Fetcher thread 1 starts with name = " + t2.getName());
        t2.start();
        System.out.println("New Fetcher thread 1 runs with name = " + t2.getName());

        System.out.println("Enqueuer thread starts with name = " + t3.getName());
    }
	
}
