/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Silvano Squizzato <silvano.squizzato@pd.infn.it>
 * 
 */

package org.glite.ce.cream.cmdmanagement.queue.db;

import org.glite.ce.common.db.DatabaseException;
import org.glite.ce.cream.cmdmanagement.db.CommandDBManager;
import org.glite.ce.cream.cmdmanagement.db.CommandTestUtil;
import org.glite.ce.creamapi.cmdmanagement.Command;
import org.glite.ce.creamapi.cmdmanagement.CommandException;
import org.glite.ce.creamapi.cmdmanagement.queue.CommandQueueException;


/**
 * Class to test command in the dB command queue.
 */
public final class CommandQueueTest {
   
	public static void main(String[] args) throws Exception {
	    test1(); 
	}
	
    /**
     * Tests operations in a command queue.
     * - enqueue
     * - enqueue
     * - dequeue
     * - fetch
     * @throws DatabaseException
     * @throws IllegalArgumentException
     * @throws CommandQueueException 
     * @throws CommandException 
     */
    private static void test1() throws DatabaseException, IllegalArgumentException, CommandQueueException, CommandException {
        CommandDBManager cmdManager = new CommandDBManager();
        CommandQueueDBManager queueManager = new CommandQueueDBManager ("JOB_MANAGEMENT");
        
        queueManager.recoverQueue();
        
        Command cmd = CommandTestUtil.getCommand("userX");
        cmdManager.insertCommand(cmd);
        System.out.println("Inserted command with cmdId = " + cmd.getId());
        
        queueManager.enqueueCommand(cmd);
        String cmdId = cmd.getId();
        System.out.println("Enqueued command with cmdId = " + cmdId);
        
        cmd = CommandTestUtil.getCommand("userX");
        cmdManager.insertCommand(cmd);
        queueManager.enqueueCommand(cmd);
        String cmdId1 = cmd.getId();
        System.out.println("Enqueued command with cmdId = " + cmdId1);
        
        queueManager.dequeueCommand(cmdId);
        System.out.println("Deleted command with cmdId = " + cmdId);
        
        Command fetchedCmd = queueManager.fetchCommand();
        System.out.println("Fetched command with cmdId = " + fetchedCmd.getId());
        
        queueManager.recoverQueue();
    }
}
