/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 * 
 * Authors: Silvano Squizzato <silvano.squizzato@pd.infn.it>
 * 
 */

package org.glite.ce.cream.cmdmanagement.queue.db;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.glite.ce.common.db.DatabaseException;
import org.glite.ce.cream.cmdmanagement.db.CommandDBManager;
import org.glite.ce.cream.jobmanagement.command.JobCancelCmd;
import org.glite.ce.cream.jobmanagement.db.JobDBImplementation;
import org.glite.ce.creamapi.cmdmanagement.CommandException;
import org.glite.ce.creamapi.cmdmanagement.queue.CommandQueueException;
import org.glite.ce.creamapi.jobmanagement.JobStatus;
import org.glite.ce.creamapi.jobmanagement.db.JobDBInterface;


/**
 * Class to test command in the dB command queue.
 */
public final class CommandQueueTest1 {
   
	public static void main(String[] args) throws Exception {
	    test1(); 
	}
	
    /**
     * Tests operations in a command queue.
     * - enqueue
     * - enqueue
     * - dequeue
     * - fetch
     * @throws DatabaseException
     * @throws IllegalArgumentException
     * @throws CommandQueueException 
     * @throws CommandException 
     */
    private static void test1() throws DatabaseException, IllegalArgumentException, CommandQueueException, CommandException {
        CommandDBManager cmdManager = new CommandDBManager();
        CommandQueueDBManager queueManager = new CommandQueueDBManager ("JOB_MANAGEMENT");
        
        List<String> jobIdList = new ArrayList<String>();
        
        JobDBInterface jobDB = new JobDBImplementation();
        jobDB.init(null);
        int[] jobStatusType = new int[3];
        jobStatusType[0] = JobStatus.IDLE;
        jobStatusType[1] = JobStatus.RUNNING;
        jobStatusType[2] = JobStatus.REALLY_RUNNING;
        jobIdList = jobDB.retrieveJobId(jobStatusType, null);
        
        
        Iterator<String> it = jobIdList.iterator();
        while (it.hasNext()) {
            String jobId = it.next();
            System.out.println("jobId = " + jobId);
            JobCancelCmd jcc = new JobCancelCmd();
            
            setJobCancelCmd(jcc, jobId);
            cmdManager.insertCommand(jcc);
            System.out.println("Inserted command with cmdId = " + jcc.getId());
            
            queueManager.enqueueCommand(jcc);
            String cmdId = jcc.getId();
            System.out.println("Enqueued command with cmdId = " + cmdId);
        }
        System.out.println("The end");
    }

    private static void setJobCancelCmd(JobCancelCmd jcc, String jobId) {
        if (jcc != null && jobId != null) {
            ArrayList<String> jobIdList = new ArrayList<String>();
            jobIdList.add(jobId);
            jcc.setJobIdList(jobIdList);
            jcc.setUserId("C_IT_O_INFN_OU_Personal_Certificate_L_Padova_CN_Luigi_Zangrando_Email_zangrando_pd.infn.it_infngrid_Role_NULL_Capability_NULL");
            jcc.setAuthNProxyPath("'/opt/glite/var/cream/user_proxy/C_IT_O_INFN_OU_Personal_Certificate_L_Padova_CN_Luigi_Zangrando_Email_zangrando_pd.infn.it_infngrid_Role_NULL_Capability_NULL/authN.proxy'");
            jcc.setDelegationProxyId("deleg5");
            jcc.setDelegationProxyPath("'/opt/glite/var/cream/user_proxy/C_IT_O_INFN_OU_Personal_Certificate_L_Padova_CN_Luigi_Zangrando_Email_zangrando_pd.infn.it_infngrid_Role_NULL_Capability_NULL/deleg_deleg.proxy'");
        }
    }
}
