/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 *
 * Authors: Eric Frizziero <eric.frizziero@pd.infn.it>
 * 
 */

package org.glite.ce.cream.jobmanagement.db;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;

import org.glite.ce.common.db.DatabaseException;
import org.glite.ce.common.db.DatasourceManager;
import org.glite.ce.cream.jobmanagement.db.table.JobCommandTable;
import org.glite.ce.creamapi.jobmanagement.command.JobCommand;
import org.glite.ce.creamapi.jobmanagement.db.table.JobCommandTableInterface;

public class JobCommandTableTest {
	String datasourceName = null;
	JobCommandTableInterface   jobCommandTable   = null;
	
	public JobCommandTableTest()throws Exception{
	  this.jobCommandTable   = new JobCommandTable();
	}
	
	  public void executeInsertTest(String jobId, String userId) throws DatabaseException{
		  int rowInserted, totRowInserted = 0;
		  JobCommand jobCommand = null;
		  List<JobCommand> jobCommandList = new ArrayList<JobCommand>();
		  
		  for (int index = 0; index<9; index++){
			  jobCommand = new JobCommand(index, jobId);
			  jobCommand.setUserId(userId);
			  jobCommand.setJobId(jobId);
			  jobCommand.setType(0);
			  jobCommandList.add(jobCommand);
		  }
		  Connection connection = null;
		  try{
			connection = DatasourceManager.getConnection(this.datasourceName);
			rowInserted = jobCommandTable.executeInsert(jobCommandList, connection);
		    totRowInserted += rowInserted;
		    connection.commit();
		  }catch (SQLException sqle){
			  System.out.println("Exception: " + sqle.getMessage());
		  }finally {
			   if (connection != null) {
			       try {
			           connection.close();
			       } catch (SQLException sqle) {
			           //do nothing
			       }
			   } 
		  } 
		  System.out.println("TotRowInserted = " + totRowInserted);
	  }
	  
	  public List<String> executeSelectTest(List<String> jobId, String userId, Calendar startDate, Calendar endDate) throws DatabaseException{
		  Connection connection = null;
		  List<String> jobIdRetrieved = null;
		  try{
			connection = DatasourceManager.getConnection(this.datasourceName);
			jobIdRetrieved = jobCommandTable.executeSelectToRetrieveJobIdByDate(jobId, userId, startDate, endDate, connection);
		  }catch (SQLException sqle){
			  System.out.println("Exception: " + sqle.getMessage());
		  }finally {
			   if (connection != null) {
			       try {
			           connection.close();
			       } catch (SQLException sqle) {
			           //do nothing
			       }
			   } 
		  } 
		  return jobIdRetrieved;
	  }
	  
	 public int executeDeleteTest(String jobId) throws DatabaseException{
		 Connection connection = null;
		 int rowDeleted = -1;
		  try{
			  connection = DatasourceManager.getConnection(this.datasourceName);
		      rowDeleted = jobCommandTable.executeDelete(jobId, connection);
		      connection.commit();
		  }catch (SQLException sqle){
			  System.out.println("Exception: " + sqle.getMessage());
		  }finally {
			   if (connection != null) {
			       try {
			           connection.close();
			       } catch (SQLException sqle) {
			           //do nothing
			       }
			   } 
		  }    
		 return rowDeleted;
	 }
	 
	 
	  public static void main(String[] args) throws Exception{
		  JobCommandTableTest jobCommandTableTest = new JobCommandTableTest();
		  
		  String jobId = "CREAM506445737";
		  String userId = "pippo";
		  //TEST INSERT
		  jobCommandTableTest.executeInsertTest(jobId, userId);
		  
		  List<String> jobIdList = new ArrayList<String>();
		  jobIdList.add(jobId);
		  List<String> jobIdListRetrieved = null;
		  Calendar startDate = Calendar.getInstance();
		  startDate.set(2007, 10, 10, 12, 10, 10);
		  
		  Calendar endDate = Calendar.getInstance();
		  startDate.set(2007, 11, 10, 12, 10, 10);
			  
			  
		  //TEST select by date
		  jobIdListRetrieved = jobCommandTableTest.executeSelectTest(jobIdList, userId, startDate, endDate);
		  for (int i=0; i<jobIdListRetrieved.size();i++){
			  System.out.println("JobId = " + jobIdListRetrieved.get(i));
		  }
		  
		  //TEST delete
		 // int rowDeleted = jobCommandTableTest.executeDeleteTest(jobId);
		 // System.out.println("rowDeleted must be 9 ... = " + rowDeleted);
	  }
	  
}

