/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

/*
 *
 * Authors: Eric Frizziero <eric.frizziero@pd.infn.it>
 * 
 */

package org.glite.ce.cream.jobmanagement.db;

import java.sql.Connection;
import java.sql.SQLException;

import org.glite.ce.common.db.DatabaseException;
import org.glite.ce.common.db.DatasourceManager;
import org.glite.ce.cream.jobmanagement.db.table.JobTable;
import org.glite.ce.creamapi.jobmanagement.Job;
import org.glite.ce.creamapi.jobmanagement.db.table.JobTableInterface;

public class JobTableTest {
	JobTableInterface   jobTable   = null;
	String datasourceName = null;
	
	public JobTableTest() throws Exception{
		this.jobTable   = new JobTable();
	}
	
	public Job createJobTest(String userId, String delegationId){
	  Job job = new Job("XXX");
	  job.setUserId(userId);
	  job.setDelegationProxyId(delegationId);
	  //Job.setCreamURL("creamurl");
	  job.setBatchSystem("BatchSystem");
	  job.setQueue("Queue");
	  job.setExecutable("Executable");
	  job.setType("Type");
	  job.setJDL("JDL");
	  job.setLocalUser("LocalUser");
	  job.setWorkingDirectory("WorkingDirectory");
	  
	  
	return job;
  }
  
  public String executeInsertTest(int numJob) throws DatabaseException{
	  int rowInserted, totRowInserted = 0;
	  Job job = null;
	  Connection connection = null;
	  try{
		  connection = DatasourceManager.getConnection(this.datasourceName);
	  for (int index = 0; index<numJob; index++){
	    //job = createJobTest("Pippo" + index + " - " + Calendar.getInstance().toString(), "Dog");
		  job = createJobTest("Pippo", "Dog");
		    
		  rowInserted = jobTable.executeInsert(job, connection);
	    totRowInserted += rowInserted;
	  }
	  connection.commit();
	  System.out.println("TotRowInserted = " + totRowInserted);
	  }catch (SQLException sqle){
		  System.out.println("Exception: " + sqle.getMessage());
	  }finally {
		   if (connection != null) {
		       try {
		           connection.close();
		       } catch (SQLException sqle) {
		           //do nothing
		       }
		   } 
	  } 
	  //return last jobId
	  return job.getId();
  }
  
  public void executeDeleteTest(String jobId) throws DatabaseException{
	  int totRowDeleted = 0;
	  Connection connection = null;
	  try{
		  connection = DatasourceManager.getConnection(this.datasourceName);
	      totRowDeleted = jobTable.executeDelete(jobId, connection);
	      connection.commit();
	      System.out.println("TotRowDeleted = " + totRowDeleted);
	  }catch (SQLException sqle){
		  System.out.println("Exception: " + sqle.getMessage());
	  }finally {
		   if (connection != null) {
		       try {
		           connection.close();
		       } catch (SQLException sqle) {
		           //do nothing
		       }
		   } 
	  } 
  }
  
 /*
  public List<String> executeSelectTest(String userId)throws SQLException {
	 return this.jobTable.executeSelect(userId);
  }
  */
  
  public static void main(String[] args) throws Exception{
	    JobTableTest jobTableTest = new JobTableTest();
	    String jobId = jobTableTest.executeInsertTest(1);
	  
	    jobTableTest.executeDeleteTest(jobId);
	 /*
	  List<String> idList = jobTableTest.executeSelectTest("Pippo");
	  if (idList.size()>0){
		 for (int i=0; i<idList.size(); i++){
			 System.out.println("ID JOB = " + idList.get(i));
		 }
	  } else {
	  System.out.println("No Job for " + "Pippo");
	  }
	  
	  */
  }
  
}
