/*
 * Copyright (c) Members of the EGEE Collaboration. 2004. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 *
 *     http://www.apache.org/licenses/LICENSE-2.0 
 *
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */

package org.glite.ce.cream.jobmanagement.serialization;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutput;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import org.apache.log4j.Logger;
import org.glite.ce.cream.jobmanagement.util.JobSerialization;
import org.glite.ce.creamapi.jobmanagement.Job;
import org.glite.ce.creamapi.jobmanagement.JobStatus;
import org.glite.ce.creamapi.jobmanagement.Lease;
import org.glite.ce.creamapi.jobmanagement.command.JobCommand;

/**
 * Test class for <code>Job</code> serialization.
 *
 */
public class SerializationTest {
    
    private static Logger logger = Logger.getLogger(SerializationTest.class.getName());
    
    /**
     * Tests <code>Job</code> serialization to file.
     * @throws Exception
     */
    /**
     * @throws Exception
     */
    public static void  testSerializationToFile() throws Exception {
        FileOutputStream fos = new FileOutputStream("t.tmp");
        ObjectOutputStream oos = new ObjectOutputStream(fos);
        Job job = JobSerialization.getDummyJob();
        
        logger.debug("Before serialization");
        String s1 = JobSerialization.printJob(job);
        logger.debug(" Job print:\n" + s1);
        
        job.writeExternal(oos);
        
        oos.close();
        fos.close();
        
        logger.debug("After deserialization");
        
        Job jobDeserialized = new Job();
        FileInputStream fis = new FileInputStream("t.tmp");
        ObjectInputStream ois = new ObjectInputStream(fis);
        jobDeserialized.readExternal(ois);
        
        String s2 = JobSerialization.printJob(jobDeserialized);
        logger.debug(" Job print:\n" + s2);
        ois.close();
        fis.close();
        
        if (s1.toString().equals(s2.toString())) {
            logger.debug("Job Serialized == Job Deserialized ");
        } else {
            logger.debug("Job Serialized != Job Deserialized ");
        }
    }
    
    /**
     * @param args The arguments are necessary.
     */
    public static void main(String[] args) {
        try {
            testSerializationToFile();
        } catch (Exception e) {
            logger.error(e);
        }
    }
}
